@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1999-2024, University of Amsterdam
+    Copyright (c)  1999-2025, University of Amsterdam
                               VU University Amsterdam
                               SWI-Prolog Solutions b.v.
 
@@ -51,12 +51,15 @@
             tbacktrace/1,               % +ThreadId,
             tbacktrace/2                % +ThreadId, +Options
           ]).
-:- if((   current_predicate(win_open_console/5)
-      ;   current_predicate('$open_xterm'/5))).
-:- export(( thread_run_interactor/0,    % interactor main loop
-            interactor/0,
+:- if(current_prolog_flag(xpce, true)).
+:- export(( interactor/0,
             interactor/1                % ?Title
           )).
+:- autoload(library(epilog),
+            [ epilog/1,
+              epilog_attach/1,
+              ep_has_console/1
+            ]).
 :- endif.
 
 :- meta_predicate
@@ -173,12 +176,13 @@ stopped_except :-
 %
 %   @see attach_console/0
 
-:- dynamic
-    has_console/4.                  % Id, In, Out, Err
-
-thread_has_console(main) :- !.                  % we assume main has one.
+thread_has_console(main) :-
+    !,
+    \+ current_prolog_flag(epilog, true).
+:- if(current_predicate(ep_has_console/1)).
 thread_has_console(Id) :-
-    has_console(Id, _, _, _).
+    ep_has_console(Id).
+:- endif.
 
 thread_has_console :-
     current_prolog_flag(break_level, _),
@@ -188,62 +192,6 @@ thread_has_console :-
     thread_has_console(Id),
     !.
 
-%!  open_console(+Title, -In, -Out, -Err) is det.
-%
-%   Open a new console window and unify In,  Out and Err with the input,
-%   output and error streams for the new console. This predicate is only
-%   available  if  win_open_console/5  (Windows  or   Qt  swipl-win)  or
-%   '$open_xterm'/5 (POSIX systems with pseudo terminal support).
-
-:- multifile xterm_args/1.
-:- dynamic   xterm_args/1.
-
-:- if(current_predicate(win_open_console/5)).
-
-can_open_console.
-
-open_console(Title, In, Out, Err) :-
-    thread_self(Id),
-    regkey(Id, Key),
-    win_open_console(Title, In, Out, Err,
-                     [ registry_key(Key)
-                     ]).
-
-regkey(Key, Key) :-
-    atom(Key).
-regkey(_, 'Anonymous').
-
-:- elif(current_predicate('$open_xterm'/5)).
-
-%!  xterm_args(-List) is nondet.
-%
-%   Multifile and dynamic hook that  provides (additional) arguments for
-%   the xterm(1) process opened  for   additional  thread consoles. Each
-%   solution must bind List to a list   of  atomic values. All solutions
-%   are concatenated using append/2 to form the final argument list.
-%
-%   The defaults set  the  colors   to  black-on-light-yellow,  enable a
-%   scrollbar, set the font using  Xft   font  pattern  and prepares the
-%   back-arrow key.
-
-xterm_args(['-xrm', '*backarrowKeyIsErase: false']).
-xterm_args(['-xrm', '*backarrowKey: false']).
-xterm_args(['-fa', 'Ubuntu Mono', '-fs', 12]).
-xterm_args(['-fg', '#000000']).
-xterm_args(['-bg', '#ffffdd']).
-xterm_args(['-sb', '-sl', 1000, '-rightbar']).
-
-can_open_console :-
-    getenv('DISPLAY', _),
-    absolute_file_name(path(xterm), _XTerm, [access(execute)]).
-
-open_console(Title, In, Out, Err) :-
-    findall(Arg, xterm_args(Arg), Args),
-    append(Args, Argv),
-    '$open_xterm'(Title, In, Out, Err, Argv).
-
-:- endif.
-
 %!  attach_console is det.
 %!  attach_console(?Title) is det.
 %
@@ -257,35 +205,17 @@ attach_console :-
 attach_console(_) :-
     thread_has_console,
     !.
-:- if(current_predicate(open_console/4)).
+:- if(current_predicate(epilog_attach/1)).
 attach_console(Title) :-
-    can_open_console,
-    !,
-    thread_self(Id),
-    (   var(Title)
-    ->  console_title(Id, Title)
-    ;   true
-    ),
-    open_console(Title, In, Out, Err),
-    assert(has_console(Id, In, Out, Err)),
-    set_stream(In,  alias(user_input)),
-    set_stream(Out, alias(user_output)),
-    set_stream(Err, alias(user_error)),
-    set_stream(In,  alias(current_input)),
-    set_stream(Out, alias(current_output)),
-    enable_line_editing(In,Out,Err),
-    thread_at_exit(detach_console(Id)).
+    thread_self(Me),
+    console_title(Me, Title),
+    epilog_attach([ title(Title)
+                  ]).
 :- endif.
 attach_console(Title) :-
     print_message(error, cannot_attach_console(Title)),
     fail.
 
-:- if(current_predicate(open_console/4)).
-console_title(Thread, Title) :-         % uses tabbed consoles
-    current_prolog_flag(console_menu_version, qt),
-    !,
-    human_thread_id(Thread, Id),
-    format(atom(Title), 'Thread ~w', [Id]).
 console_title(Thread, Title) :-
     current_prolog_flag(system_thread_id, SysId),
     human_thread_id(Thread, Id),
@@ -299,40 +229,6 @@ human_thread_id(Thread, Alias) :-
 human_thread_id(Thread, Id) :-
     thread_property(Thread, id(Id)).
 
-%!  enable_line_editing(+In, +Out, +Err) is det.
-%
-%   Enable line editing for the console.  This   is  by built-in for the
-%   Windows console. We can also provide it   for the X11 xterm(1) based
-%   console if we use the BSD libedit based command line editor.
-
-enable_line_editing(_In, _Out, _Err) :-
-    current_prolog_flag(readline, editline),
-    exists_source(library(editline)),
-    use_module(library(editline)),
-    !,
-    call(el_wrap).
-enable_line_editing(_In, _Out, _Err).
-
-disable_line_editing(_In, _Out, _Err) :-
-    current_predicate(el_unwrap/1),
-    !,
-    call(el_unwrap(user_input)).
-disable_line_editing(_In, _Out, _Err).
-
-
-%!  detach_console(+ThreadId) is det.
-%
-%   Destroy the console for ThreadId.
-
-detach_console(Id) :-
-    (   retract(has_console(Id, In, Out, Err))
-    ->  disable_line_editing(In, Out, Err),
-        close(In, [force(true)]),
-        close(Out, [force(true)]),
-        close(Err, [force(true)])
-    ;   true
-    ).
-
 %!  interactor is det.
 %!  interactor(?Title) is det.
 %
@@ -342,57 +238,21 @@ detach_console(Id) :-
 interactor :-
     interactor(_).
 
+:- if(current_predicate(epilog/1)).
 interactor(Title) :-
-    current_prolog_flag(epilog, true),
     !,
     (   nonvar(Title)
     ->  Options = [title(Title)]
     ;   Options = []
     ),
-    autoload_call(epilog(Options)).
-interactor(Title) :-
-    can_open_console,
-    !,
-    thread_self(Me),
-    thread_create(thread_run_interactor(Me, Title), _Id,
-                  [ detached(true)
-                  ]),
-    thread_get_message(Msg),
-    (   Msg = title(Title0)
-    ->  Title = Title0
-    ;   Msg = throw(Error)
-    ->  throw(Error)
-    ;   Msg = false
-    ->  fail
-    ).
+    epilog([ init(true)
+           | Options
+           ]).
+:- endif.
 interactor(Title) :-
     print_message(error, cannot_attach_console(Title)),
     fail.
 
-thread_run_interactor(Creator, Title) :-
-    set_prolog_flag(query_debug_settings, debug(false, false)),
-    Error = error(Formal,_),
-    (   catch(attach_console(Title), Error, true)
-    ->  (   var(Formal)
-        ->  thread_send_message(Creator, title(Title)),
-            print_message(banner, thread_welcome),
-            prolog
-        ;   thread_send_message(Creator, throw(Error))
-        )
-    ;   thread_send_message(Creator, false)
-    ).
-
-%!  thread_run_interactor
-%
-%   Attach a console and run a Prolog toplevel in the current thread.
-
-thread_run_interactor :-
-    set_prolog_flag(query_debug_settings, debug(false, false)),
-    attach_console(_Title),
-    print_message(banner, thread_welcome),
-    prolog.
-
-:- endif.                               % have open_console/4
 
                  /*******************************
                  *          DEBUGGING           *
@@ -545,13 +405,12 @@ init_pce.
                  *******************************/
 
 :- multifile
-    user:message_hook/3.
+    prolog:message_action/2.
 
-user:message_hook(trace_mode(on), _, Lines) :-
+prolog:message_action(trace_mode(on), _Level) :-
     \+ thread_has_console,
     \+ current_prolog_flag(gui_tracer, true),
-    catch(attach_console, _, fail),
-    print_message_lines(user_error, '% ', Lines).
+    catch(attach_console, error(_,_), fail).
 
 :- multifile
     prolog:message/3.
@@ -569,7 +428,7 @@ prolog:message(joined_threads(Threads)) -->
 prolog:message(threads(Threads)) -->
     thread_list(Threads).
 prolog:message(cannot_attach_console(_Title)) -->
-    [ 'Cannot attach a console (requires swipl-win or POSIX pty support)' ].
+    [ 'Cannot attach a console (requires xpce package)' ].
 
 thread_list(Threads) -->
     { maplist(th_id_len, Threads, Lens),
