@@ -1,10 +1,9 @@
 /*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi-prolog.org
-    WWW:           https://www.swi-prolog.org
-    Copyright (c)  2000-2026, University of Amsterdam
-			      SWI-Prolog Solutions b.v.
+    E-mail:        J.Wielemaker@vu.nl
+    WWW:           http://www.swi-prolog.org
+    Copyright (c)  2000-2023, University of Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -50,9 +49,6 @@
 #include "form.h"
 #ifdef __WINDOWS__
 #include <io.h>
-#ifdef _MSC_VER
-typedef intptr_t ssize_t;
-#endif
 #define read _read
 #define fileno _fileno
 #endif
@@ -64,9 +60,11 @@ Breaks a string holding data from a WWW form into its values.  Outputs a
 sequence of NAME=VALUE commands for a shell.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static char /* simplified */
-dehex(char chr)
-{ if ( chr >= '0' && chr <= '9' )
+static int
+dehex(int chr)
+{ chr &= 0xff;
+
+  if ( chr >= '0' && chr <= '9' )
     return chr - '0';
   if ( chr >= 'A' && chr <= 'F' )
     return chr - 'A' + 10;
@@ -90,8 +88,8 @@ form_argument_decode(const char *in, size_t inlen, char *out, size_t outlen)
         break;
       case '%':
 	if ( in+2 < ein )
-	{ char h1 = dehex(*(++in));
-	  char h2 = dehex(*(++in));
+	{ int h1 = dehex(*(++in));
+	  int h2 = dehex(*(++in));
 
 	  if ( h1 < 0 || h2 < 0 )
 	    return (size_t)-1;
@@ -117,7 +115,7 @@ form_argument_decode(const char *in, size_t inlen, char *out, size_t outlen)
 
 #define SHORTVALUE 512
 
-int // bool or ERROR_*
+int
 break_form_argument(const char *formdata,
 		    int (*func)(const char* name,
 				size_t namelen,
@@ -172,7 +170,7 @@ break_form_argument(const char *formdata,
     }
   }
 
-  return true;
+  return TRUE;
 }
 
 
@@ -258,15 +256,15 @@ next_line(const char *in)
 }
 
 
-bool
+int
 break_multipart(char *formdata, size_t len,
 		const char *boundary,
-		bool (*func)(const char *name,
-			     size_t namelen,
-			     const char *value,
-			     size_t valuelen,
-			     const char *filename,
-			     void *closure),
+		int (*func)(const char *name,
+			    size_t namelen,
+			    const char *value,
+			    size_t valuelen,
+			    const char *filename,
+			    void *closure),
 		void *closure)
 { char *enddata = formdata+len;
 
@@ -311,10 +309,10 @@ break_multipart(char *formdata, size_t len,
     end[0] = '\0';
 
     if ( !(func)(name, strlen(name), data, end-data, filename, closure) )
-      return false;
+      return FALSE;
   }
 
-  return true;
+  return TRUE;
 }
 
 
@@ -324,8 +322,8 @@ provided, it is filled with the length  of the contents. The input value
 for lenp is the maximum acceptable content-length.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-bool
-get_raw_form_data(char **data, size_t *lenp, bool *must_free)
+int
+get_raw_form_data(char **data, size_t *lenp, int *must_free)
 { char *method;
   char *s;
 
@@ -333,7 +331,7 @@ get_raw_form_data(char **data, size_t *lenp, bool *must_free)
        strcmp(method, "POST") == 0 )
   { char *lenvar = getenv("CONTENT_LENGTH");
     char *q;
-    ssize_t len;
+    long len;
 
     if ( !lenvar )
     { term_t env = PL_new_term_ref();
@@ -345,8 +343,8 @@ get_raw_form_data(char **data, size_t *lenp, bool *must_free)
     if ( len < 0 )
     { term_t t = PL_new_term_ref();
 
-      if ( !PL_put_int64(t, len) )
-	return false;
+      if ( !PL_put_integer(t, len) )
+	return FALSE;
       return pl_error(NULL, 0, "< 0", ERR_DOMAIN, t, "content_length");
     }
     if ( lenp )
@@ -354,8 +352,8 @@ get_raw_form_data(char **data, size_t *lenp, bool *must_free)
       { term_t t = PL_new_term_ref();
 	char msg[100];
 
-	if ( !PL_put_int64(t, len) )
-	  return false;
+	if ( !PL_put_integer(t, len) )
+	  return FALSE;
 	snprintf(msg, sizeof msg, "> %ld", (long)*lenp);
 
 	return pl_error(NULL, 0, msg, ERR_DOMAIN, t, "content_length");
@@ -367,15 +365,9 @@ get_raw_form_data(char **data, size_t *lenp, bool *must_free)
     if ( !q )
       return pl_error(NULL, 0, NULL, ERR_RESOURCE, "memory");
     while(len > 0)
-    { size_t done;
+    { int done;
 
-#ifdef __WINDOWS__
-#define RDLEN(len) ((unsigned int)len)
-#else
-#define RDLEN(len) len
-#endif
-
-      while( (done=read(fileno(stdin), q, RDLEN(len))) > 0 )
+      while( (done=read(fileno(stdin), q, len)) > 0 )
       { q+=done;
 	len-=done;
       }
@@ -395,16 +387,16 @@ get_raw_form_data(char **data, size_t *lenp, bool *must_free)
     if ( len == 0 )
     { *q = '\0';
       *data = s;
-      *must_free = true;
-      return true;
+      *must_free = TRUE;
+      return TRUE;
     } else
       goto no_data;
   } else if ( (s = getenv("QUERY_STRING")) )
   { if ( lenp )
       *lenp = strlen(s);
     *data = s;
-    *must_free = false;
-    return true;
+    *must_free = FALSE;
+    return TRUE;
   } else
   { term_t env = PL_new_term_ref();
     PL_put_atom_chars(env, "QUERY_STRING");
