@@ -138,8 +138,8 @@ PREDICATE(hello3, 2)
   // character in it. In addition, a NUL ('\0') in the atom will cause
   // the rest of the atom to not be printed.
 
-  int len = Ssnprintf(buf, sizeof buf,
-		      "Hello3 %Ws\n", atom_a1.as_wstring().c_str());
+  int len = SsnprintfX(buf, sizeof buf,
+		       "Hello3 %Ws\n", atom_a1.as_wstring().c_str());
   if ( len >= 0 )
     // TODO: use len when fixed: https://github.com/SWI-Prolog/swipl-devel/issues/1074
     return A2.unify_chars(PL_STRING|REP_UTF8, strlen(buf), buf);
@@ -221,7 +221,7 @@ PREDICATE(list_modules, 1)
   PlTermv av(1);
 
   PlQuery q("current_module", av);
-  while( q.next_solution() )
+  while( PlWrap<int>(q.next_solution()) )
     buffer << av[0].as_string() << endl;
 
   q.cut();
@@ -239,7 +239,7 @@ PREDICATE(average, 3)			/* average(+Templ, :Goal, -Average) */
      PlQuery q("call", av);
   */
   PlQuery q("call", PlTermv(A2));
-  while( q.next_solution() )
+  while( PlWrap<int>(q.next_solution()) )
   { sum += A1.as_long();
     n++;
   }
@@ -249,7 +249,7 @@ PREDICATE(average, 3)			/* average(+Templ, :Goal, -Average) */
 
 PREDICATE(call_cpp, 2)
 { PlQuery q(A1.as_string(), PlTermv(A2));
-  PlCheckFail(q.next_solution());
+  PlCheckFail(PlWrap<int>(q.next_solution()));
   // There's no need for calling q.cut() - it's done implicitly by the
   // query's destructor.
   return true;
@@ -257,7 +257,7 @@ PREDICATE(call_cpp, 2)
 
 PREDICATE(call_cut, 1)
 { PlQuery q(A1.as_string(), PlTermv());
-  PlCheckFail(q.next_solution());
+  PlCheckFail(PlWrap<int>(q.next_solution()));
   q.cut(); // This tests that ~PlQuery() behaves correctly if cut() had been called
   return true;
 }
@@ -1401,7 +1401,36 @@ PREDICATE(cvt_i_bool, 2)
 
 // TODO: add tests for PL_cvt_i_*() (using PlTerm::integer())
 
-// TODO: add PlEngine tests
+int fake_main(int argc, char *argv[]) // TODO: add test that calls this.
+{ PlEngine e(argc, argv);
+  PlTermv av(1);
+  PlTerm_tail l(av[0]);
+
+  for(int i=0; i<argc; i++)
+    PlCheckFail(l.append(PlTerm_string(argv[i])));
+  PlCheckFail(l.close());
+
+  PlQuery q("entry", av);
+  try
+  { return PlWrap<int>(q.next_solution()) ? 0 : 1;
+  } catch ( PlException &ex )
+  { std::cerr << ex.what() << std::endl;
+    return 1;
+  }
+}
+
+int fake_main_legacy(int argc, char **argv) // TODO: add test that calls this.
+{ PlEngine e(argc, argv);
+  PlTermv av(1);
+  PlTerm_tail l(av[0]);
+
+  for(int i=0; i<argc; i++)
+    PlCheckFail(l.append(PlTerm_string(argv[i])));
+  PlCheckFail(l.close());
+
+  PlQuery q("entry", av);
+  return PlWrap<int>(q.next_solution()) ? 0 : 1;
+}
 
 PREDICATE(throw_domain_cpp0, 1)
 { return Plx_domain_error("footype", A1.unwrap());
@@ -1531,7 +1560,7 @@ struct MyConnection
 
 
 // The following code is taken from
-// pl2cpp2.doc \subsubsection{Sample PlBlob code}
+// pl2cpp.doc \subsubsection{Sample PlBlob code}
 // with some minor changes for testing
 
 struct MyBlob;
@@ -1551,7 +1580,7 @@ struct MyBlob : public PlBlob
       connection(std::make_unique<MyConnection>(connection_name)),
       blob_name(connection_name)
   { if ( !connection ) // make_unique should have thrown exception if it can't allocate
-      PL_api_error("MyBlob(%s) connection=%p", blob_name.c_str(), connection.get());
+      PL_api_error("MyBlob(%s) connection=%p", blob_name.c_str(), static_cast<const void*>(connection.get()));
     if ( !connection->open() )
       throw MyBlobError("my_blob_open_error");
     if ( name_contains("FAIL_new") ) // Test error handling
@@ -1650,7 +1679,7 @@ PREDICATE(portray_my_blob, 2)
 
 
 // TODO: Add the following code to
-// pl2cpp2.doc \subsubsection{Sample PlBlob code (pointer)}
+// pl2cpp.doc \subsubsection{Sample PlBlob code (pointer)}
 
 struct MyFileBlob;
 
@@ -2141,9 +2170,10 @@ NAMED_PREDICATE("#", hash, 2)
 }
 
 PREDICATE(malloc_free, 2)
-{ // For verifying that example code of unique_ptr with PLfree() compiles
+{ // For verifying that example code of unique_ptr with PL_free() compiles
   std::unique_ptr<void, decltype(&PL_free)> ptr(PL_malloc(100), &PL_free);
 
+  PlStringBuffers _string_buffers;
   size_t len;
   char *str = nullptr;
   int rc = Plx_get_nchars(A1.unwrap(), &len, &str, BUF_MALLOC|CVT_ALL|CVT_WRITEQ|CVT_VARIABLE|REP_UTF8|CVT_EXCEPTION);
@@ -2173,7 +2203,7 @@ PREDICATE(free_blob, 1)
 
 PREDICATE(nil_repr, 1)
 { char buf[100];
-  snprintf(buf, sizeof buf, "%p", nullptr);
+  snprintf(buf, sizeof buf, "%p", static_cast<const void*>(nullptr));
   return A1.unify_string(buf);
 }
 
