@@ -227,9 +227,7 @@ help(P) :->
 prolog_manual(Topic) :-
     clause(prolog_help:show_html_hook(_), _),
     !,
-    \+ \+ ( b_setval(prolog_help_gui, true),
-            help(Topic)
-          ).
+    help(Topic).
 prolog_manual(Topic) :-
     current_prolog_flag(epilog, true),
     !,
