@@ -36,7 +36,6 @@
 #include <h/kernel.h>
 #include <h/graphics.h>
 
-static status	defaultPostScriptFont(FontObj f);
 static Int	getPointsFont(FontObj f);
 static status	loadFontFamilies(void);
 static status	loadFontAliases(Name res);
@@ -89,9 +88,10 @@ initialiseFont(FontObj f, Name family, Name style, Int points, Name weight)
   assign(f, style,       style);
   assign(f, weight,	 weight);
   assign(f, points,      points);
+  assign(f, ascent,      DEFAULT);
+  assign(f, descent,     DEFAULT);
   assign(f, fixed_width, DEFAULT);
 
-  defaultPostScriptFont(f);
   protectObject(f);		/* Still needed? */
 
   Name name = fontName(family, style, points, weight);
@@ -142,6 +142,22 @@ getConvertFont(Class class, Name name)
   fail;
 }
 
+FontObj
+getCopyFont(FontObj f)
+{ FontObj copy = allocObject(classOfObject(f), FALSE);
+
+  ws_create_font(f);
+  assign(copy, family,      f->family);
+  assign(copy, style,       f->style);
+  assign(copy, weight,	    f->weight);
+  assign(copy, points,      f->points);
+  assign(copy, ascent,      f->ascent);
+  assign(copy, descent,     f->descent);
+  assign(copy, fixed_width, f->fixed_width);
+  copy->ws_ref = ws_clone_ws_font(f->ws_ref);
+
+  answer(copy);
+}
 
 static FontObj
 getRescaleFont(FontObj f, Num scale)
@@ -197,48 +213,6 @@ makeBuiltinFonts(void)
   fail;
 }
 
-		/********************************
-		*          POSTSCRIPT		*
-		********************************/
-
-static status
-defaultPostScriptFont(FontObj f)
-{ char buf[LINESIZE];
-
-  if ( f->family == NAME_helvetica )
-  { strcpy(buf, "Helvetica");
-
-    if ( f->style == NAME_bold )
-      strcat(buf, "-Bold");
-    else if ( f->style == NAME_oblique )
-      strcat(buf, "-Oblique");
-  } else if ( f->family == NAME_times )
-  { strcpy(buf, "Times");
-
-    if ( f->style == NAME_bold )
-      strcat(buf, "-Bold");
-    else if ( f->style == NAME_italic )
-      strcat(buf, "-Italic");
-    else /*if ( f->style == NAME_roman )*/
-      strcat(buf, "-Roman");
-  } else if ( f->style == NAME_ansiVar )
-  { strcpy(buf, "Helvetica");
-  } else				/* default */
-  { strcpy(buf, "Courier");
-
-    if ( f->style == NAME_bold )
-      strcat(buf, "-Bold");
-    else if ( f->style == NAME_oblique )
-      strcat(buf, "-Oblique");
-  }
-
-  assign(f, postscript_size, getPointsFont(f));
-  assign(f, postscript_font, CtoName(buf));
-
-  succeed;
-}
-
-
 		/********************************
 		*           GET INFO		*
 		********************************/
@@ -285,19 +259,22 @@ getAvgCharWidthFont(FontObj f)
 
 Num
 getHeightFont(FontObj f)
-{ answer(toNum(s_height(f)));
+{ ws_create_font(f);
+  answer(toNum(valNum(f->ascent)+valNum(f->descent)));
 }
 
 
 Num
 getAscentFont(FontObj f)
-{ answer(toNum(s_ascent(f)));
+{ ws_create_font(f);
+  answer(f->ascent);
 }
 
 
 Num
 getDescentFont(FontObj f)
-{ answer(toNum(s_descent(f)));
+{ ws_create_font(f);
+  answer(f->descent);
 }
 
 
@@ -491,16 +468,16 @@ static vardecl var_font[] =
      NAME_name, "Weight of the font"),
   IV(NAME_points, "[num]", IV_NONE,
      NAME_name, "Point-size of the font"),
+  IV(NAME_ascent, "[num]", IV_NONE,
+     NAME_dimension, "Height above baseline"),
+  IV(NAME_descent, "[num]", IV_NONE,
+     NAME_dimension, "Depth below baseline"),
   IV(NAME_ex, "num*", IV_NONE,
      NAME_dimension, "Height of the letter `x' in this font"),
   IV(NAME_avgCharWidth, "num*", IV_NONE,
      NAME_dimension, "Average char width"),
   IV(NAME_fixedWidth, "[bool]", IV_NONE,
-     NAME_property, "If @off, font is proportional"),
-  IV(NAME_postscriptFont, "name", IV_BOTH,
-     NAME_postscript, "PostScript-name of the font"),
-  IV(NAME_postscriptSize, "int", IV_BOTH,
-     NAME_postscript, "PostScript point-size of the font"),
+     NAME_dimension, "If @off, font is proportional"),
   IV(NAME_wsRef, "alien:WsRef", IV_NONE,
      NAME_storage, "Window system handle")
 };
