@@ -3145,8 +3145,7 @@ drawImageGraphical(Graphical gr, Image img,
 	  isDefault(sy) ? 0 : valInt(sy),
 	  valInt(x), valInt(y),
 	  isDefault(sw) ? valInt(img->size->w) : valInt(sw),
-	  isDefault(sh) ? valInt(img->size->h) : valInt(sh),
-	  transparent);
+	  isDefault(sh) ? valInt(img->size->h) : valInt(sh));
 
   succeed;
 }
@@ -3165,7 +3164,7 @@ drawTextGraphical(Graphical gr, CharArray txt, FontObj font,
 
     str_string(&txt->data, font,
 	       valInt(x), valInt(y), valInt(w), valInt(h),
-	       hadjust, vadjust, 0);
+	       hadjust, vadjust, OFF, 0);
   }
 
   succeed;
