@@ -1,9 +1,10 @@
 /*  Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        jan@swi.psy.uva.nl
-    WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (c)  1985-2002, University of Amsterdam
+    E-mail:        jan@swi-prolog.org
+    WWW:           https://www.swi-prolog.org/projects/xpce/
+    Copyright (c)  1985-2025, University of Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -61,8 +62,6 @@ initialiseImage(Image image, SourceSink data, Int w, Int h, Name kind)
     name = NIL;
 
   assign(image, name,       name);
-  assign(image, background, DEFAULT);
-  assign(image, foreground, DEFAULT);
   ws_init_image(image);
 
   if ( isNil(data) || notDefault(w) || notDefault(h) || notDefault(kind) )
@@ -72,13 +71,11 @@ initialiseImage(Image image, SourceSink data, Int w, Int h, Name kind)
 
     assign(image, kind,   kind);
     assign(image, file,   NIL);
-    assign(image, depth,  toInt(32));
     assign(image, size,	  newObject(ClassSize, w, h, EAV));
     assign(image, access, NAME_both);
   } else
   { assign(image, kind,	  NAME_pixmap);
     assign(image, file,	  data);
-    assign(image, depth,  toInt(32));
     assign(image, size,	  newObject(ClassSize, EAV));
     TRY(loadImage(image, DEFAULT, DEFAULT));
     assign(image, access, NAME_read);
@@ -271,11 +268,9 @@ XopenImage(Image image, DisplayObj d)
 { if ( image->bits )			/* built-in.  See stdImage() */
   { switch(image->bits->type)
     { case XBM_DATA:
-	ws_create_image_from_x11_data(image,
-				      image->bits->bits.xbm,
-				      valInt(image->size->w),
-				      valInt(image->size->h));
-	break;
+	if ( image->bits->bits.xbm )	/* NULL image has no data */
+	  Cprintf("XopenImage(%s)\n", pp(image));
+        break;
       case XPM_DATA:
 	ws_create_image_from_xpm_data(image,
 				      image->bits->bits.xpm,
@@ -284,18 +279,17 @@ XopenImage(Image image, DisplayObj d)
       default:
 	assert(0);
     }
-			/* Windows already does the registration */
     if ( getExistingXrefObject(image, d) )
       succeed;
   }
 
-  return ws_open_image(image, d, valReal(image->scale));
+  return ws_open_image(image);
 }
 
 
 status
 XcloseImage(Image image, DisplayObj d)
-{ ws_close_image(image, d);
+{ ws_close_image(image);
 
   succeed;
 }
@@ -352,11 +346,6 @@ verifyAccessImage(Image image, Name sel)
 { if ( image->access != NAME_both )
     return errorPce(image, NAME_readOnly);
 
-  if ( isNil(image->display) )
-    assign(image, display, CurrentDisplay(image));
-
-  openDisplay(image->display);
-
   succeed;
 }
 
@@ -397,8 +386,7 @@ clearImage(Image image)
 
   CHANGING_IMAGE(image,
 		 if ( image->size->w != ZERO && image->size->h != ZERO &&
-		      notNil(image->display) &&
-		      getExistingXrefObject(image, image->display) != NULL )
+		      image->ws_ref != NULL )
 		 { int w = valInt(image->size->w);
 		   int h = valInt(image->size->h);
 
@@ -413,21 +401,13 @@ clearImage(Image image)
 }
 
 
-static status
-scaleImage(Image image, Real factor)
-{ assign(image, scale, factor);
-  if ( notNil(image->mask) )
-    assign(image->mask, scale, factor);
-  /* TBD: Verify the image is not yet opened */
-  succeed;
-}
-
 static status
 resizeImage(Image image, Int w, Int h)
 { TRY( verifyAccessImage(image, NAME_resize) );
 
   CHANGING_IMAGE(image,
-		 ws_resize_image(image, w, h));
+		 assign(image->size, w, w);
+		 assign(image->size, h, h));
 
   succeed;
 }
@@ -445,7 +425,7 @@ copyImage(Image image, Image i2)
 
     if ( d_image(image, 0, 0, valInt(w), valInt(h)) )
     { d_modify();
-      r_image(i2, 0, 0, 0, 0, valInt(w), valInt(h), OFF);
+      r_image(i2, 0, 0, 0, 0, valInt(w), valInt(h));
       d_done();
       changedEntireImageImage(image);
     });
@@ -602,24 +582,6 @@ pixelImage(Image image, Int X, Int Y, Any val)
 }
 
 
-static status
-setPixelImage(Image image, Int x, Int y)
-{ if ( image->kind == NAME_bitmap )
-    return pixelImage(image, x, y, ON);
-  else
-    return pixelImage(image, x, y, image->foreground);
-}
-
-
-static status
-clearPixelImage(Image image, Int x, Int y)
-{ if ( image->kind == NAME_bitmap )
-    return pixelImage(image, x, y, OFF);
-  else
-    return pixelImage(image, x, y, image->background);
-}
-
-
 static status
 invertPixelImage(Image image, Int x, Int y)
 { TRY(verifyAccessImage(image, NAME_invertPixel));
@@ -666,7 +628,7 @@ getPixelImage(Image image, Int x, Int y)
       if ( pixel == NoPixel )
 	result = FAIL;
       else
-	result = ws_pixel_to_colour(image->display, pixel);
+	result = ws_pixel_to_colour(pixel);
     }
     d_done();
 
@@ -677,19 +639,6 @@ getPixelImage(Image image, Int x, Int y)
 }
 
 
-static status
-maskImage(Image image, Image mask)
-{ assign(image, mask, mask);
-
-  if ( notNil(image->bitmap) )
-    updateSolidBitmap(image->bitmap);
-
-  ws_prepare_image_mask(image);
-
-  return changedEntireImageImage(image);
-}
-
-
 		/********************************
 		*      LOGICAL OPERATIONS	*
 		********************************/
@@ -762,13 +711,11 @@ getClipImage(Image image, Area area)
     if ( hx >= 0 && hx <= w && hy >= 0 && hy <= h )
       assign(i2, hot_spot, newObject(ClassPoint, toInt(hx), toInt(hy), EAV));
   }
-  if ( notNil(image->mask) )
-    assign(i2, mask, getClipImage(image->mask, area));
 
   CHANGING_IMAGE(i2,
     d_image(i2, 0, 0, w, h);
     d_modify();
-    r_image(image, x, y, 0, 0, w, h, OFF);
+    r_image(image, x, y, 0, 0, w, h);
     d_done();
     changedEntireImageImage(i2););
 
@@ -802,13 +749,6 @@ getScaleImage(Image image, Size size)
 
   i2 = ws_scale_image(image, valInt(size->w), valInt(size->h));
 
-  if ( notNil(image->mask) )
-  { Image m = getScaleImage(image->mask, size);
-
-    if ( m )
-      assign(i2, mask, m);
-  }
-
   if ( notNil(image->hot_spot) )
   { int hx = (valInt(image->hot_spot->x) * valInt(size->w)) /
 						    valInt(image->size->w);
@@ -822,14 +762,15 @@ getScaleImage(Image image, Size size)
 
 
 static Image
-getRotateImage(Image image, Real degrees)
-{ float a = (float)valReal(degrees);
+getRotateImage(Image image, Num degrees)
+{ double a = valNum(degrees);
   Image rimg;
 
-  a -= (float)(((int)a / 360)*360);
-  if ( a < 0.0 )				/* normalise 0<=a<360 */
+  a = fmod(a, 360.0);
+  if ( a < 0 )
     a += 360.0;
-  else if ( a == 0.0 )				/* just copy */
+
+  if ( a < 0.01 )				/* just copy */
     answer(getClipImage(image, DEFAULT));
 
   rimg = ws_rotate_image(image, a);
@@ -858,9 +799,6 @@ getRotateImage(Image image, Real degrees)
 
       assign(rimg, hot_spot, newObject(ClassPoint, toInt(nhx), toInt(nhy), EAV));
     }
-
-    if ( notNil(image->mask) )
-      assign(rimg, mask, getRotateImage(image->mask, degrees));
   }
 
   answer(rimg);
@@ -895,11 +833,6 @@ stdImage(Name name, Image *global, unsigned char *bits, int w, int h)
   return image;
 }
 
-static inline int
-rescale(Image image, int px)
-{ return (int)((double)px*valReal(image->scale)+0.5);
-}
-
 #ifdef XPM_PCEIMAGE
 static void
 stdXPMImage(Name name, Name kind, Image *global, char **bits)
@@ -909,16 +842,13 @@ stdXPMImage(Name name, Name kind, Image *global, char **bits)
   { Image image = globalObject(name, ClassImage, name, toInt(w), toInt(h), EAV);
 
     if ( colours == 2 )
-    { assign(image, depth, ONE);
-      assign(image, kind, kind);
+    { assign(image, kind, kind);
     } else
     { assign(image, kind, NAME_pixmap);
     }
 
     assign(image, access, NAME_read);
-    setSize(image->size,
-	    toInt(rescale(image, w)),
-	    toInt(rescale(image, h)));
+    setSize(image->size, toInt(w), toInt(h));
     image->bits = alloc(sizeof(*image->bits));
     image->bits->type = XPM_DATA;
     image->bits->bits.xpm = bits;
@@ -942,27 +872,9 @@ stdXPMImage(Name name, Name kind, Image *global, char **bits)
 #include "bitmaps/nomark.xpm"
 #endif
 
-#include "bitmaps/white_bm"
-#include "bitmaps/grey50_bm"
-#include "bitmaps/black_bm"
-
-static void
-greyImage(Name name, int grey, Image *global,
-	  unsigned char *bits, int w, int h)
-{ stdImage(name, global, bits, w, h);
-}
-
-
 static void
 standardImages(void)
-{ greyImage(NAME_whiteImage,  0,  &WHITE_IMAGE,
-	    white_bm_bits, white_bm_width, white_bm_height);
-  greyImage(NAME_grey50Image, 50, &GREY50_IMAGE,
-	    grey50_bm_bits, grey50_bm_width, grey50_bm_height);
-  greyImage(NAME_blackImage, 100, &BLACK_IMAGE,
-	    black_bm_bits, black_bm_width, black_bm_height);
-
-  stdImage(NAME_msLeftArrowImage, NULL,
+{ stdImage(NAME_msLeftArrowImage, NULL,
 	   ms_left_arrow_bits, ms_left_arrow_width, ms_left_arrow_height);
   stdImage(NAME_markHandleImage, &MARK_HANDLE_IMAGE,
 	   mark_handle_bm_bits, mark_handle_bm_width, mark_handle_bm_height);
@@ -991,8 +903,6 @@ standardImages(void)
 
   stdImage(NAME_nullImage, &NULL_IMAGE,
 	   NULL, 0, 0);
-
-  ws_system_images();			/* make sure system images exist */
 }
 
 
@@ -1021,10 +931,6 @@ static char *T_xAint_yAint[] =
 	{ "x=int", "y=int" };
 static char *T_pixel[] =
 	{ "x=int", "y=int", "value=colour|bool" };
-#ifdef O_XLI
-static char *T_loadXli[] =
-	{ "file=file", "bright=[0..]" };
-#endif
 
 /* Instance Variables */
 
@@ -1037,24 +943,12 @@ static vardecl var_image[] =
      NAME_file, "Source (file,resource) from which to load"),
   IV(NAME_access, "{read,both}", IV_GET,
      NAME_permission, "One of {read, both}"),
-  IV(NAME_background, "[colour]*", IV_BOTH,
-     NAME_colour, "Colour of background"),
-  IV(NAME_foreground, "[colour]", IV_BOTH,
-     NAME_colour, "Colour of foreground"),
-  IV(NAME_depth, "[int]", IV_GET,
-     NAME_colour, "Number of bits/pixel"),
   IV(NAME_size, "size", IV_GET,
      NAME_dimension, "Size of the image in pixels"),
-  SV(NAME_scale, "real", IV_GET|IV_STORE, scaleImage,
-     NAME_dimension, "Set scale factor for image"),
-  IV(NAME_display, "display*", IV_GET,
-     NAME_organisation, "X-Display this image belongs to"),
   IV(NAME_bitmap, "bitmap*", IV_GET,
      NAME_organisation, "Access both and displayed on this bitmap"),
   IV(NAME_hotSpot, "point*", IV_BOTH,
      NAME_dimension, "Hot-spot position"),
-  SV(NAME_mask, "image*", IV_GET|IV_STORE, maskImage,
-     NAME_area, "Image for masked painting"),
   IV(NAME_wsRef, "alien:WsRef", IV_NONE,
      NAME_storage, "Window System Reference"),
   IV(NAME_bits, "alien:char*", IV_NONE,
@@ -1063,10 +957,6 @@ static vardecl var_image[] =
 
 /* Send Methods */
 
-#ifdef O_XLI
-extern status loadXliImage(Image image, FileObj file, Int bright);
-#endif
-
 static senddecl send_image[] =
 { SM(NAME_initialise, 4, T_initialise, initialiseImage,
      DEFAULT, "Create from name, [width, height, kind]"),
@@ -1094,20 +984,12 @@ static senddecl send_image[] =
      NAME_edit, "Bitwise xor with argument"),
   SM(NAME_load, 2, T_load, loadImage,
      NAME_file, "Load image from file (searching in path)"),
-#ifdef O_XLI
-  SM(NAME_loadXli, 2, T_loadXli, loadXliImage,
-     NAME_file, "Load image using xli library"),
-#endif
   SM(NAME_save, 2, T_save, saveImage,
      NAME_file, "Save image to file in specified format"),
-  SM(NAME_clearPixel, 2, T_xAint_yAint, clearPixelImage,
-     NAME_pixel, "Clear pixel at x-y (to 0 or background)"),
   SM(NAME_invertPixel, 2, T_xAint_yAint, invertPixelImage,
      NAME_pixel, "Invert pixel at x-y"),
   SM(NAME_pixel, 3, T_pixel, pixelImage,
      NAME_pixel, "Set pixel at x-y to bool or colour"),
-  SM(NAME_setPixel, 2, T_xAint_yAint, setPixelImage,
-     NAME_pixel, "Set pixel at x-y (to 1 or foreground)"),
   SM(NAME_Xclose, 1, "display", XcloseImage,
      NAME_x, "Destroy associated window-system resources"),
   SM(NAME_Xopen, 1, "display", XopenImage,
@@ -1129,7 +1011,7 @@ static getdecl get_image[] =
      NAME_copy, "Get grayscale version of image"),
   GM(NAME_scale, 1, "image", "size", getScaleImage,
      NAME_copy, "Get copy with different dimensions"),
-  GM(NAME_rotate, 1, "image", "degrees=real", getRotateImage,
+  GM(NAME_rotate, 1, "image", "degrees=num", getRotateImage,
      NAME_copy, "Get anti-clockwise rotated copy"),
   GM(NAME_lookup, 1, "image", "name|resource", getLookupImage,
      NAME_oms, "Lookup in @images table"),
@@ -1143,9 +1025,7 @@ static classvardecl rc_image[] =
 { RC(NAME_path, "string",
      "\".:bitmaps:~/lib/bitmaps:$PCEHOME/bitmaps:" /* concat */
      "/usr/include/X11/bitmaps\"",
-     "Search path for loading images"),
-  RC(NAME_scale, "real", "1.0",
-     "Scale factor for the image")
+     "Search path for loading images")
 };
 
 /* Class Declaration */
