@@ -101,7 +101,7 @@ RedrawLabelDialogItem(Any obj, int acc,
     else
       iy = y + h-ih;
 
-    r_image(i, 0, 0, ix, iy, iw, ih, ON);
+    r_image(i, 0, 0, ix, iy, iw, ih);
   } else if ( instanceOfObject(di->label, ClassCharArray) )
   { CharArray label = di->label;
 
