@@ -44,8 +44,6 @@ static PangoFontMap *fontmap;	/* Per surface type (screen, PDF, ... */
 static PangoContext *context;	/* Per DPI and fontmap */
 static double font_scale = 1.0;
 
-#define P2D(i) ((i)/(double)PANGO_SCALE)
-
 static void clean_width_cache(charwidth_cache *wcache);
 
 static status
@@ -67,6 +65,30 @@ ws_init_fonts(void)
 
 #define TRUST_PANGO_METRICS 0
 
+static void
+set_ex_font(FontObj f)
+{ WsFont wsf = f->ws_ref;
+  PangoGlyphString *glyphs = pango_glyph_string_new();
+  PangoAnalysis analysis = {0};
+  PangoRectangle ink;
+  const char *sample = "x";
+  double ex;
+
+  analysis.font = wsf->font;
+  pango_shape(sample, -1, &analysis, glyphs);
+
+  if ( glyphs->num_glyphs == 1 )
+  { PangoGlyphInfo *gi = &glyphs->glyphs[0];
+    pango_font_get_glyph_extents(wsf->font, gi->glyph, &ink, NULL);
+
+    ex = P2D(ink.height);
+  } else
+  { ex = (valNum(f->ascent) + valNum(f->descent))/2.0; /* approximated "x" */
+  }
+
+  assign(f, ex, toNum(ex));
+}
+
 #if !TRUST_PANGO_METRICS
 
 static void
@@ -99,8 +121,8 @@ dynamic_metrics(FontObj f)
 		pp(f), P2D(max_ascent), P2D(max_descent)));
 
   pango_glyph_string_free(glyphs);
-  wsf->ascent  = P2D(max_ascent)  + 1.0;
-  wsf->descent = P2D(max_descent) + 1.0;
+  assign(f, ascent,  toNum(P2D(max_ascent)  + 1.0));
+  assign(f, descent, toNum(P2D(max_descent) + 1.0));
 }
 
 #endif/*TRUST_PANGO_METRICS*/
@@ -197,14 +219,15 @@ ws_create_font(FontObj f)
   f->ws_ref = wsf;
 
 #if TRUST_PANGO_METRICS
-  wsf->ascent  = pango_font_metrics_get_ascent(metrics)/(double)PANGO_SCALE;
-  wsf->descent = pango_font_metrics_get_descent(metrics)/(double)PANGO_SCALE;
+  wsf->ascent  = P2D(pango_font_metrics_get_ascent(metrics));
+  wsf->descent = P2D(pango_font_metrics_get_descent(metrics));
 #else
   dynamic_metrics(f);
 #endif
 
-  wsf->height  = wsf->ascent + wsf->descent;
-  assign(f, ex, toNum(wsf->height/2.0)); /* approximated height of "x" */
+  wsf->ul_thickness = P2D(pango_font_metrics_get_underline_thickness(metrics));
+  wsf->ul_position  = P2D(pango_font_metrics_get_underline_position(metrics));
+  set_ex_font(f);
   pango_font_metrics_unref(metrics);
 
   succeed;
