@@ -1671,16 +1671,13 @@ rlc_paint_text(RlcData b,
 	  Cprintf("No bold font\n");
       }
 
+      //Cprintf("Print \"%s\" at %d,%d using %s\n", t, *cx, ty, pp(ti->font));
       int x0 = *cx;
       *cx += tchar_width(b, t, ulen, segment, ti->font);
-#if 0
-      fprintf(stderr, "Print \"%s\"[%d] at %d[%d],%d using %s\n",
-	      t, ulen, x0, *cx-x0, ty, pp(ti->font));
-#endif
       r_clear(x0, ty-b->cb, *cx-x0, b->ch);
       s_print_utf8(t, ulen, x0, ty, font);
       if ( TF_UNDERLINE(flags) )
-	r_underline(font, x0, ty, *cx-x0, DEFAULT);
+	r_underline(font, x0, ty, *cx, DEFAULT);
       if ( TF_INVERSE(flags) )
 	r_swap_background_and_foreground();
       if ( notDefault(ofg) )
@@ -2595,12 +2592,6 @@ rlc_sgr(RlcData b, int sgr)
   }
 }
 
-static void
-rlc_24bit_colour(RlcData b, bool fg, int red, int green, int blue)
-{ DEBUG(NAME_term, Cprintf("Colour: %s %d,%d,%d (not implemented)\n",
-			   fg ? "fg" : "bg", red, green, blue));
-}
-
 static RlcTextLine
 rlc_prepare_line(RlcData b, int y)
 { RlcTextLine tl = &b->lines[b->caret_y];
@@ -2677,8 +2668,8 @@ rlc_check_links(RlcTextLine tl)
       { Cprintf("CHECK: %03d could not find href for %d(%d); got: ",
 		tl->line_no, start, len);
 	for(hr = tl->links; hr; hr=hr->next)
-	  Cprintf(" %d(%d)", hr->start, hr->length);
-	Cprintf("\n");
+	  Dprintf(_T(" %d(%d)"), hr->start, hr->length);
+	Dprintf(_T("\n"));
       }
     }
   }
@@ -2687,7 +2678,7 @@ rlc_check_links(RlcTextLine tl)
   for(href *hr = tl->links; hr; hr=hr->next)
     refs++;
   if ( refs != links )
-  { Cprintf("CHECK: %03d found %d links; expected %d\n",
+  { Dprintf("CHECK: %03d found %d links; expected %d\n",
 	    tl->line_no, links, refs);
     Dprint_line(tl, true);
   }
@@ -2979,7 +2970,7 @@ rlc_put_link(RlcData b, const uchar_t *label, const uchar_t *link)
 }
 
 #ifdef _DEBUG
-#define CMD(c) do {cmd = #c; c;} while(0)
+#define CMD(c) do {cmd = _T(#c); c;} while(0)
 #else
 #define CMD(c) do {c;} while(0)
 #endif
@@ -2988,7 +2979,7 @@ static void
 rlc_putansi(RlcData b, int chr)
 {
 #ifdef _DEBUG
-  const char *cmd;
+  uchar_t *cmd;
 #endif
 
   switch(b->cmdstat)
@@ -3279,16 +3270,11 @@ rlc_putansi(RlcData b, int chr)
 	  CMD(rlc_erase_line(b));
 	  break;
 	case 'm':
-	  { rlc_need_arg(b, 1, 0);
+	  { int i;
+	    rlc_need_arg(b, 1, 0);
 
-	    if ( (b->argv[0] == 38 || b->argv[0] == 48) &&
-		 b->argc == 5 && b->argv[1] == 2 )
-	    { CMD(rlc_24bit_colour(b, b->argv[0] == 38,
-				   b->argv[2], b->argv[3], b->argv[4]));
-	    } else
-	    { for(int i=0; i<b->argc; i++)
-		CMD(rlc_sgr(b, b->argv[i]));
-	    }
+	    for(i=0; i<b->argc; i++)
+	      CMD(rlc_sgr(b, b->argv[i]));
 	    break;
 	  }
 	case '@':		/* \e[<n>@: insert <n> spaces */
