@@ -38,21 +38,22 @@
 #include <h/text.h>
 #include <h/unix.h>
 
-static Int    getMatchingQuoteTextBuffer(TextBuffer, Int, Name);
-static status capitalise_textbuffer(TextBuffer, intptr_t, intptr_t);
-static status clear_textbuffer(TextBuffer);
-static status downcase_textbuffer(TextBuffer, intptr_t, intptr_t);
-static void   end_change(TextBuffer, intptr_t);
-static Int    getSizeTextBuffer(TextBuffer);
-static status store_textbuffer(TextBuffer, intptr_t, wint_t);
-static status transpose_textbuffer(TextBuffer, intptr_t, intptr_t, intptr_t, intptr_t);
-static status upcase_textbuffer(TextBuffer, intptr_t, intptr_t);
-static status save_textbuffer(TextBuffer, intptr_t, intptr_t, SourceSink);
-static status insert_file_textbuffer(TextBuffer, intptr_t, intptr_t, SourceSink);
-static status shift_fragments(TextBuffer, intptr_t, intptr_t);
-static void   start_change(TextBuffer, intptr_t);
-static status insert_textbuffer_shift(TextBuffer, intptr_t, intptr_t, PceString, int);
-static status promoteTextBuffer(TextBuffer tb);
+forwards Int	getMatchingQuoteTextBuffer(TextBuffer, Int, Name);
+forwards int	room(TextBuffer, intptr_t, intptr_t);
+forwards status capitalise_textbuffer(TextBuffer, intptr_t, intptr_t);
+forwards status clear_textbuffer(TextBuffer);
+forwards status downcase_textbuffer(TextBuffer, intptr_t, intptr_t);
+forwards void	end_change(TextBuffer, intptr_t);
+forwards Int    getSizeTextBuffer(TextBuffer);
+forwards status store_textbuffer(TextBuffer, intptr_t, wint_t);
+forwards status transpose_textbuffer(TextBuffer, intptr_t, intptr_t, intptr_t, intptr_t);
+forwards status upcase_textbuffer(TextBuffer, intptr_t, intptr_t);
+forwards status save_textbuffer(TextBuffer, intptr_t, intptr_t, SourceSink);
+forwards status insert_file_textbuffer(TextBuffer, intptr_t, intptr_t, SourceSink);
+forwards status shift_fragments(TextBuffer, intptr_t, intptr_t);
+forwards void	start_change(TextBuffer, intptr_t);
+forwards status insert_textbuffer_shift(TextBuffer, intptr_t, intptr_t, PceString, int);
+forwards status promoteTextBuffer(TextBuffer tb);
 
 #define ALLOC (256)		/* increment allocation by this amount */
 #define ROUND(n, r)		( (((n) + (r)-1) / (r)) * (r) )
@@ -2020,9 +2021,13 @@ update_lsp_pos_text_buffer(TextBuffer tb, size_t from, size_t to, lsp_pos *pos)
 	  { pos->line++;
 	    pos->pos = 0;
 	  } else
-	  { pos->pos++;
+	  {
+#if SIZEOF_WCHAR_T == 2
+	    pos->pos++;
+#else
 	    if ( b[i] > 0xffff )
 	      pos->pos++;
+#endif
 	  }
 	}
 	b = tb->tb_bufferW + tb->gap_end - tb->gap_start;
@@ -2240,7 +2245,7 @@ transpose_textbuffer(TextBuffer tb, intptr_t f1, intptr_t t1, intptr_t f2, intpt
 
   register_change_textbuffer(tb, f1, t2-f1);
 
-  room_text_buffer(tb, t2, 0);			/* move gap out of the way */
+  room(tb, t2, 0);			/* move gap out of the way */
   t1--; t2--;
   mirror_textbuffer(tb, f1, t2);
   mirror_textbuffer(tb, f1, f1+t2-f2);
@@ -2317,7 +2322,7 @@ static status
 save_textbuffer(TextBuffer tb, intptr_t from, intptr_t len, SourceSink file)
 { IOSTREAM *fd;
 
-  room_text_buffer(tb, tb->size, 0);		/* move the gap to the end */
+  room(tb, tb->size, 0);		/* move the gap to the end */
 
   if ( !(fd = Sopen_object(file, "wr")) )
     return errorPce(file, NAME_openFile, NAME_write, getOsErrorPce(PCE));
@@ -2371,7 +2376,7 @@ str_sub_text_buffer(TextBuffer tb, PceString s, intptr_t start, intptr_t len)
     len = tb->size - start;
 
   if ( start < tb->gap_start && start+len > tb->gap_start )
-    room_text_buffer(tb, start + len, 1);
+    room(tb, start + len, 1);
 
   str_cphdr(s, &tb->buffer);
   s->s_size = len;
@@ -2481,7 +2486,7 @@ insert_file_textbuffer(TextBuffer tb, intptr_t where, intptr_t times, SourceSink
     fail;
   size = Ssize(fd);			/* size in bytes */
 
-  room_text_buffer(tb, where, size);		/* always enough */
+  room(tb, where, size);		/* always enough */
   where = tb->gap_start;		/* normalised */
   start_change(tb, tb->gap_start);
 
@@ -2541,7 +2546,7 @@ done:
   register_insert_textbuffer(tb, where, grow);
 
   times--;
-  room_text_buffer(tb, tb->gap_start, times*size);	/* enough for the copies */
+  room(tb, tb->gap_start, times*size);	/* enough for the copies */
   while(times-- > 0)
   { memmove(Address(tb, tb->gap_start),
 	    Address(tb, where),
@@ -2583,7 +2588,7 @@ insert_textbuffer_shift(TextBuffer tb, intptr_t where, intptr_t times,
 
   grow = times * s->s_size;
   where = NormaliseIndex(tb, where);
-  room_text_buffer(tb, where, grow);
+  room(tb, where, grow);
 
   register_insert_textbuffer(tb, where, grow);
   start_change(tb, tb->gap_start);
@@ -2680,7 +2685,7 @@ delete_textbuffer(TextBuffer tb, intptr_t where, intptr_t length)
   if ( length <= 0 )				/* out of bounds: ignore */
     succeed;
 
-  room_text_buffer(tb, where, 0);				/* move the gap here */
+  room(tb, where, 0);				/* move the gap here */
   register_delete_textbuffer(tb, where, length);
 
   start_change(tb, where);
@@ -2796,8 +2801,8 @@ end_change(TextBuffer tb, intptr_t where)
 
  ** Tue Apr  4 17:23:28 1989  jan@swivax.UUCP (Jan Wielemaker)  */
 
-void
-room_text_buffer(TextBuffer tb, intptr_t where, intptr_t grow)
+static int
+room(TextBuffer tb, intptr_t where, intptr_t grow)
 { ssize_t shift;
 
   if ( grow + tb->size > tb->allocated )
@@ -2831,6 +2836,8 @@ room_text_buffer(TextBuffer tb, intptr_t where, intptr_t grow)
   }
   tb->gap_start += shift;			/* move the gap pointers */
   tb->gap_end += shift;
+
+  succeed;
 }
 
 
